import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { DeleteResult, Repository } from 'typeorm';
import { Album } from './album.entity';
import { CreateAlbumDto } from './dto/create-album.dto';
import { UpdateAlbumDto } from './dto/update-album.dto';

@Injectable()
export class AlbumService {
  constructor(
    @InjectRepository(Album)
    private albumRepository: Repository<Album>,
  ) {}

  findAll(): Promise<Album[] | string | null> {
    return this.albumRepository.find({
      order: {
        artist: 'ASC',
        title: 'ASC',
      },
    }).then((albums) => {
      return albums;
    })
    .catch((error) => {
      return `There was a problem getting the list of albums: ${error}`
    });
  }

  findOneById(id: number): Promise<Album | string | null> {
    return this.albumRepository.findOne({
      where: {
        id: id,
      },
      relations: {
        songs: true,
      },
      order: {
        songs: {
          trackNumber: 'ASC',
        },
      },
    }).then((album) => {
      return album;
    })
    .catch((error) => {
      return `There was a problem creating the Album identified by ID ${id}: ${error}`
    }).finally(() => {
      return `There was a problem creating the Album identified by ID ${id}`
    });
  }

  async create(createAlbumDto: CreateAlbumDto): Promise<Album | string | null> {
    const album = this.albumRepository.create({
      title: createAlbumDto.title,
      artist: createAlbumDto.artist,
      genre: createAlbumDto.genre,
    });
    return await this.albumRepository.save(album)
      .then((savedAlbum) => {
        return savedAlbum;
      })
      .catch((error) => {
        return `There was a problem creating the Album (${createAlbumDto.title} by ${createAlbumDto.artist} (${createAlbumDto.genre})): ${error}`
      });
  }

  async update(
    id: number,
    updateAlbumDto: UpdateAlbumDto,
  ): Promise<Album | string | null> {
    if (id == updateAlbumDto.id) {
      const albumToUpdate = await this.albumRepository.findOneBy({
        id: updateAlbumDto.id,
      });
      if (!albumToUpdate) {
        console.error("AlbumService: update: Didn't find album: ", id);
        return null;
      }

      await this.albumRepository.update(
        { id: updateAlbumDto.id },
        {
          title: updateAlbumDto.title,
          artist: updateAlbumDto.artist,
          genre: updateAlbumDto.genre,
        },
      );
      const album = await this.albumRepository.findOneBy({
        id: updateAlbumDto.id,
      });
      return album;
    } else {
      console.error(
        'AlbumService: update: IDs do not match',
        id,
        updateAlbumDto,
      );
      return null;
    }
  }

  async remove(id: number): Promise<DeleteResult | string | null> {
    return await this.albumRepository.delete(id)
    .then((deleteResult) => {
      return deleteResult;
    })
    .catch((error) => {
      return `There was a problem deleting the Album identified by ID ${id}: ${error}`
    });
  }
}
